#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include "CisModScanU.h"


//main function to all the operation
int main (int argc, char *argv[]) 
{
	//get the default values
	if(argc==1)
	{
		PrintReadme();
		return 0;
	}

	INPUTPARAMETER* pInputParam=new INPUTPARAMETER;
	SetDefaultParam(pInputParam);


	ParseInputParam(pInputParam,argc,argv);

	//exit if w<5 or W>40
	if(pInputParam->nMinWidth<5 || pInputParam->nMaxWidth>40)
	{
		printf("input motif width range error!\n");
		exit(0);
	}
	//eixt if K<0
	if(pInputParam->nMotifNumber<1)
	{
		printf("Input number of transcription factors error!\n");
		exit(0);
	}
	//exit if L<w
	if(pInputParam->nModuleLength<pInputParam->nMinWidth)
	{
		printf("Input module length too small!\n");
		exit(0);
	}

	RunModScan(pInputParam);
	//delete input param
	delete pInputParam;

	return 0;
}

//set default p
void SetDefaultParam(INPUTPARAMETER* pInputParam)
{
	pInputParam->bReverse=true;			
	pInputParam->nMaxIteration=1;
	pInputParam->nMotifNumber=3;
	pInputParam->nMinWidth=10;
	pInputParam->nMaxWidth=14;
	pInputParam->nModuleLength=200;
	pInputParam->bMotifMode=false;
	pInputParam->bFixModuleLength=true;
	pInputParam->strInputSeq="";
	pInputParam->strOutput="Output.txt";	
	pInputParam->nRun=1;
	pInputParam->bISBF=false;
	pInputParam->dblr=1.0/2000;
}

//parse input parameters for the program
void ParseInputParam(INPUTPARAMETER* pInputParam, int argc, char* argv[])
{
	pInputParam->strInformation="\n\nCisModScan 1.0\n";
	for(int i=0;i<argc;i++)
	{
		pInputParam->strInformation=pInputParam->strInformation+argv[i]+" ";
		if(argv[i][0]=='-')
		{
			if(argv[i][1]=='i')
			{
				pInputParam->strInputSeq=argv[i+1];
			}
			else if(argv[i][1]=='o')
			{
				pInputParam->strOutput=argv[i+1];
			}
			else if(argv[i][1]=='r')
			{
				if(argv[i+1][0]=='0') pInputParam->bReverse=false;
				else pInputParam->bReverse=true;
			}
			else if(argv[i][1]=='x')
			{
				pInputParam->dblr=atof(argv[i+1]);
			}
			else if(argv[i][1]=='K')
			{
				pInputParam->nMotifNumber=atoi(argv[i+1]);
			}
			else if(argv[i][1]=='L')
			{
				pInputParam->nModuleLength=atoi(argv[i+1]);
			}
			else if(argv[i][1]=='M')	//files for PSMs
			{
				pInputParam->strPWMFile=argv[i+1];
			}
		}
	}
	pInputParam->strInformation=pInputParam->strInformation+"\n\n";
}

//one run of the module sampler
void RunModScan(INPUTPARAMETER* pInputParam)
{
	CGibbsDP ModSampler;

	//get the input parameters
	ModSampler.m_strInformation=pInputParam->strInformation;
	ModSampler.m_strInputSeq=pInputParam->strInputSeq;
	ModSampler.m_strOutput=pInputParam->strOutput;
	ModSampler.m_strPWMFile=pInputParam->strPWMFile;
	ModSampler.m_bReverse=pInputParam->bReverse;
	ModSampler.m_nMotifNumber=pInputParam->nMotifNumber;
	ModSampler.m_nModuleLength=pInputParam->nModuleLength;
	ModSampler.m_dblr=pInputParam->dblr;
	
	//sampling by DA
	ModSampler.ModuleScan();

}

void PrintReadme()
{
	printf("\n******************************\n");
	printf("   CisModScan 1.0 \n");
	printf("******************************\n");
	printf("\nUsage: ./CisModScan -i Seqfile (options)\n");
	printf("Seqfile must be in FASTA format and each line in the file should contain no more than 10K characters\n\n");
	printf("Options:\n");
	printf("\t-o\tOutput file (default output.txt)\n");
	printf("\t-r\t1: search for both strands; 0: only search for forward strand (default 1)\n");
	printf("\t-K\tNumber of transcription factors (default 3)\n");
	printf("\t-L\tModule length (default 200)\n");
	printf("\t-M\tInput text files for weight matrices\n");
	printf("\t-x\tExpected module occurrent rate (default 0.0005)\n");
}

